/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.pegsolitaire;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import com.carlosbcruz.bruteforce.Possibility;
import com.carlosbcruz.bruteforce.Problem;


/**
 * Represent one problem.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class OnlyOneProblem implements Problem {

    private static final int E = 0, // Empty position
            F = 1, // Filled position
            N = 2; // Not applied position

    private int board[][] = new int[7][7];

    private int standard[][] = { {N, N, F, F, F, N, N}, {N, N, F, F, F, N, N},
        {F, F, F, F, F, F, F}, {F, F, F, E, F, F, F}, {F, F, F, F, F, F, F},
        {N, N, F, F, F, N, N}, {N, N, F, F, F, N, N}};

    /**
     * Default constructor
     */
    public OnlyOneProblem() {

        for (int i = 0; i < 7; i++)
            for (int j = 0; j < 7; j++)
                board[i][j] = standard[i][j];
    }

    /**
     * Constructor that receives the problem file name
     * @param Arq the problem file name
     */
    public OnlyOneProblem(String Arq) {

        try {

            File fIn = new File(Arq);
            FileReader fr = new FileReader(fIn);
            BufferedReader in = new BufferedReader(fr);

            boolean eof = false;
            int numElem = 0;
            int lin = 0, col = 0;
            do {

                int inChar = in.read();
                if (inChar != -1) {
                    boolean compatible = false;

                    switch (inChar) {
                        case (' '): {
                            System.out.print("N");
                            board[lin][col] = N;
                            compatible = true;
                            break;
                        }
                        case ('O'): {
                            System.out.print("F");
                            board[lin][col] = F;
                            compatible = true;
                            break;
                        }
                        case ('.'): {
                            System.out.print("E");
                            board[lin][col] = E;
                            compatible = true;
                            break;
                        }
                        case ('\n'): {
                            System.out.println("|");
                            break;
                        }
                    }
                    if (compatible) {
                        numElem++;
                        col++;
                        if (col == 7) {
                            col = 0;
                            lin++;
                        }
                    }
                } else
                    eof = true;
            }
            while (!eof);

            in.close();

            if (numElem == 49)
                System.out.println("\nReading successful");
            else
                System.out.println("\nProblem reading");

        }
        catch(IOException e) {

            System.out.println("Error --" + e.toString());

        }
        catch(SecurityException se) {

            System.out.println("Error --" + se.toString());

        }

    }

    /**
     * Set the problem to be solved
     * @param the board
     */
    private void setProblem(int stand[][]) {

        board = new int[7][7];
        for (int i = 0; i < 7; i++)
            for (int j = 0; j < 7; j++)
                board[i][j] = stand[i][j];
    }

    /* (non-Javadoc)
     * @see bruteforce.Problem#apply(bruteforce.Possibility)
     */
    public void apply(Possibility possib) {

        OnlyOnePossibility op = (OnlyOnePossibility)possib;

        board[op.getXfrom()][op.getYfrom()] = E;
        board[op.getXto()][op.getYto()] = F;
        board[op.getXout()][op.getYout()] = E;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {

        String out = new String();

        for (int i = 0; i < 7; i++) {
            for (int j = 0; j < 7; j++) {
                switch (board[i][j]) {
                    case (E): {
                        out += ".";
                        break;
                    }
                    case (F): {
                        out += "O";
                        break;
                    }
                    case (N): {
                        out += " ";
                        break;
                    }
                }
            }
            out += "\n";
        }
        return out;
    }

    /* (non-Javadoc)
     * @see bruteforce.Problem#getPossibilities()
     */
    public Vector getPossibilities() {

        Vector possib = new Vector();

        for (int i = 0; i < 7; i++) {
            for (int j = 0; j < 7; j++) {
                if (board[i][j] == F) {
                    if (i + 2 <= 6) { // If it is inside of the board
                        if (board[i + 1][j] == F && board[i + 2][j] == E) {
                            possib.addElement(new OnlyOnePossibility(i, j,
                                i + 2, j, i + 1, j));
                        }
                    }
                    if (i - 2 >= 0) { // If it is inside of the board
                        if (board[i - 1][j] == F && board[i - 2][j] == E) {
                            possib.addElement(new OnlyOnePossibility(i, j,
                                i - 2, j, i - 1, j));
                        }
                    }
                    if (j + 2 <= 6) { // If it is inside of the board
                        if (board[i][j + 1] == F && board[i][j + 2] == E) {
                            possib.addElement(new OnlyOnePossibility(i, j, i,
                                j + 2, i, j + 1));
                        }
                    }
                    if (j - 2 >= 0) { // If it is inside of the board
                        if (board[i][j - 1] == F && board[i][j - 2] == E) {
                            possib.addElement(new OnlyOnePossibility(i, j, i,
                                j - 2, i, j - 1));
                        }
                    }
                }
            }
        }

        return possib;
    }

    /* (non-Javadoc)
     * @see bruteforce.Problem#isSolved()
     */
    public boolean isSolved() {

        boolean solved = false;

        int numElem = 0;

        for (int i = 0; i < 7; i++)
            for (int j = 0; j < 7; j++)
                if (board[i][j] == F)
                    numElem++;

        return (numElem == 1);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    public Object clone() throws CloneNotSupportedException {

        OnlyOneProblem newProb = (OnlyOneProblem)super.clone();
        newProb.setProblem(board);
        return newProb;

    }

}